@@ -21,8 +21,13 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include <algorithm>
 #include <iostream>
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
 typedef vtkDICOMVR VR;
 
 // Prototype for function that reads one query key
@@ -56,7 +61,15 @@ vtkDICOMTagPath path_append(const vtkDICOMTagPath& tpath, vtkDICOMTag tag)
 bool dicomcli_readquery(
   const char *fname, vtkDICOMItem *query, QueryTagList *ql)
 {
+#ifdef _WIN32
+  int cn = MultiByteToWideChar(CP_UTF8, 0, fname, -1, NULL, 0);
+  wchar_t *wfname = new wchar_t[cn];
+  MultiByteToWideChar(CP_UTF8, 0, fname, -1, wfname, cn);
+  ifstream f(wfname);
+  delete [] wfname;
+#else
   ifstream f(fname);
+#endif
   if (!f.good())
     {
     return false;
@@ -309,8 +322,10 @@ bool dicomcli_readkey_query(
   size_t valueStart = s;
   size_t valueEnd = s;
   bool valueContainsQuotes = false;
+  bool keyHasAssignment = false;
   if (s < n && cp[s] == '=')
     {
+    keyHasAssignment = true;
     s++;
     valueStart = s;
     valueEnd = s;
@@ -366,8 +381,12 @@ bool dicomcli_readkey_query(
   // add the tag and value to the query data set
   if (valueStart == valueEnd)
     {
-    // empty value (always matches, always retrieved)
-    query->SetAttributeValue(tagPath, vtkDICOMValue(vr));
+    // only overwrite previous value if '=' was explicitly used
+    if (keyHasAssignment || !query->GetAttributeValue(tagPath).IsValid())
+      {
+      // empty value (always matches, always retrieved)
+      query->SetAttributeValue(tagPath, vtkDICOMValue(vr));
+      }
     }
   else if (valueContainsQuotes)
     {
@@ -389,10 +408,13 @@ bool dicomcli_readkey_query(
       vtkDICOMValue(vr, &cp[valueStart], valueEnd - valueStart));
     }
 
-  // add the tag path to the list
+  // add the tag path to the list, if it isn't already there
   if (ql)
     {
-    ql->push_back(tagPath);
+    if (std::find(ql->begin(), ql->end(), tagPath) == ql->end())
+      {
+      ql->push_back(tagPath);
+      }
     }
 
   return !tagError;
@@ -478,3 +500,97 @@ bool dicomcli_looks_like_key(const char *cp)
 
   return false;
 }
+
+bool dicomcli_readuids(
+  const char *fname, vtkDICOMItem *query, QueryTagList *ql)
+{
+#ifdef _WIN32
+  int cn = MultiByteToWideChar(CP_UTF8, 0, fname, -1, NULL, 0);
+  wchar_t *wfname = new wchar_t[cn];
+  MultiByteToWideChar(CP_UTF8, 0, fname, -1, wfname, cn);
+  ifstream f(wfname);
+  delete [] wfname;
+#else
+  ifstream f(fname);
+#endif
+  if (!f.good())
+    {
+    return false;
+    }
+
+  // Basic file structure:
+  // # one or more comments
+  // GGGG,EEEE   # a tag or key (only one)
+  // 1.185.234   # a UID, followed by more UIDs
+
+  QueryTagList ql2;
+  std::string val;
+  int lineNumber = 0;
+  while (f.good())
+    {
+    std::string line;
+    std::getline(f, line);
+    const char *cp = line.c_str();
+    size_t n = line.size();
+    lineNumber++;
+
+    // strip leading whitespace
+    size_t s = 0;
+    while (s < n && isspace(cp[s]))
+      {
+      s++;
+      }
+
+    // skip trailing whitespace
+    if (n > s && isspace(cp[n-1]))
+      {
+      --n;
+      }
+
+    // skip line if it is a comment
+    if (s == n || cp[s] == '#')
+      {
+      continue;
+      }
+
+    if (ql2.size() == 0)
+      {
+      // get the tag line, if not gotten yet
+      if (!dicomcli_readkey_query(cp, query, &ql2, true))
+        {
+        fprintf(stderr, "Error %s line %d: ", fname, lineNumber);
+        fprintf(stderr, "Need Valid DICOM tag at top of file.\n");
+        return false;
+        }
+      }
+    else
+      {
+      // read and append a value
+      if (val.length() > 0)
+        {
+        val.append("\\", 1);
+        }
+      val.append(&cp[s], n-s);
+      if (val.length() >= 65535)
+        {
+        fprintf(stderr, "Error %s line %d: ", fname, lineNumber);
+        fprintf(stderr, "Too many values (there is a 65535 byte limit)\n");
+        return false;
+        }
+      }
+    }
+
+  if (ql2.size() > 0)
+    {
+    // add the key and value to the query
+    vtkDICOMTagPath tagPath = ql2[0];
+    query->SetAttributeValue(tagPath, val);
+
+    if (ql && std::find(ql->begin(), ql->end(), tagPath) == ql->end())
+      {
+      ql->push_back(tagPath);
+      }
+    }
+
+  return true;
+}
