@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2016 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -24,6 +24,7 @@
 #include "vtkDICOMFileDirectory.h"
 
 // from dicomcli
+#include "vtkConsoleOutputWindow.h"
 #include "mainmacro.h"
 #include "readquery.h"
 #include "progress.h"
@@ -37,7 +38,6 @@
 #include <ctype.h>
 
 #include <limits>
-#include <iostream>
 #include <map>
 #include <utility>
 
@@ -46,7 +46,7 @@ void dicompull_version(FILE *file, const char *cp)
 {
   fprintf(file, "%s %s\n", cp, DICOM_VERSION);
   fprintf(file, "\n"
-    "Copyright (c) 2012-2015, David Gobbi.\n\n"
+    "Copyright (c) 2012-2016, David Gobbi.\n\n"
     "This software is distributed under an open-source license.  See the\n"
     "Copyright.txt file that comes with the vtk-dicom source distribution.\n");
 }
@@ -61,6 +61,7 @@ void dicompull_usage(FILE *file, const char *cp)
     "  -P              Do not follow symbolic links.\n"
     "  -k tag=value    Provide an attribute to be queried and matched.\n"
     "  -q <query.txt>  Provide a file to describe the find query.\n"
+    "  -u <uids.txt>   Provide a file that contains a list of UIDs.\n"
     "  -o <directory>  Directory to place the files into.\n"
     "  -maxdepth n     Set the maximum directory depth.\n"
     "  -name pattern   Set file names to match (with \"*\" or \"?\").\n"
@@ -100,6 +101,10 @@ void dicompull_help(FILE *file, const char *cp)
     "The values used for \"-k key=value\" can use the wildcards * and ?, and\n"
     "can also use date ranges of the form \"19990103-19990105\".\n"
     "\n"
+    "The \"-u\" option can be used to provide a list of UIDs.  Provide a file\n"
+    "where the first line of the file is the key (e.g. SeriesInstanceUID) and\n"
+    "the rest of the file is the UIDs to match, one UID per line.\n"
+    "\n"
   );
 }
 
@@ -248,6 +253,9 @@ MAINMACRO_PASSTHROUGH(-name);
 // This program will find and copy dicom files
 int MAINMACRO(int argc, char *argv[])
 {
+  // redirect all VTK errors to stderr
+  vtkConsoleOutputWindow::Install();
+
   int rval = 0;
   int scandepth = std::numeric_limits<int>::max();
   bool followSymlinks = true;
@@ -261,6 +269,9 @@ int MAINMACRO(int argc, char *argv[])
 
   vtkSmartPointer<vtkStringArray> a = vtkSmartPointer<vtkStringArray>::New();
 
+  // always query SpecificCharacterSet
+  query.SetAttributeValue(DC::SpecificCharacterSet, vtkDICOMValue(VR::CS));
+
   // always query the functional sequences for advanced files
   query.SetAttributeValue(
     DC::SharedFunctionalGroupsSequence, vtkDICOMValue(VR::SQ));
@@ -309,6 +320,21 @@ int MAINMACRO(int argc, char *argv[])
         return 1;
         }
       }
+    else if (strcmp(arg, "-u") == 0)
+      {
+      if (argi + 1 == argc || argv[argi+1][0] == '-')
+        {
+        fprintf(stderr, "Error: %s must be followed by a file.\n\n", arg);
+        dicompull_usage(stderr, dicompull_basename(argv[0]));
+        return 1;
+        }
+      const char *qfile = argv[++argi];
+      if (!dicomcli_readuids(qfile, &query, &qtlist))
+        {
+        fprintf(stderr, "Error: Can't read uid file %s\n\n", qfile);
+        return 1;
+        }
+      }
     else if (strcmp(arg, "-k") == 0)
       {
       vtkDICOMTag tag;
