@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2016 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -504,6 +504,16 @@ size_t Encoder<E>::WriteElementHead(
     cp[7] = 0;
     Encoder<E>::PutInt32(cp+8, vl);
     }
+  else if (vl > 65534)
+    {
+    // CP-1066 allows overlength values to be written as UN
+    hl = 12;
+    cp[4] = 'U';
+    cp[5] = 'N';
+    cp[6] = 0;
+    cp[7] = 0;
+    Encoder<E>::PutInt32(cp+8, vl);
+    }
   return hl;
 }
 
@@ -1581,16 +1591,22 @@ unsigned int vtkDICOMCompiler::ComputePixelDataSize()
 
 //----------------------------------------------------------------------------
 bool vtkDICOMCompiler::FlushBuffer(
-  unsigned char* &ucp, unsigned char* &)
+  unsigned char* &ucp, unsigned char* &ep)
 {
+  bool rval = true;
   const unsigned char *cp = ucp;
   unsigned char *dp = this->Buffer;
   ucp = dp;
-  size_t n = cp - dp;
+  ep = dp + this->ChunkSize;
 
-  size_t m = this->OutputFile->Write(dp, n);
+  if (cp)
+    {
+    size_t n = cp - dp;
+    size_t m = this->OutputFile->Write(dp, n);
+    rval = (n == m);
+    }
 
-  return (n == m);
+  return rval;
 }
 
 //----------------------------------------------------------------------------
