@@ -111,6 +111,11 @@ public:
    */
   virtual void Update() { this->Update(0); }
   virtual void Update(int);
+#if (VTK_MAJOR_VERSION == 7 && VTK_MINOR_VERSION > 0) || VTK_MAJOR_VERSION > 7
+  virtual int Update(vtkInformation *) { this->Update(); return 1; }
+  virtual int Update(int i, vtkInformationVector *) {
+    this->Update(i); return 1; }
+#endif
   //@}
 
   //@{
@@ -186,7 +191,7 @@ public:
   //@}
 
   //@{
-  //! If On (the default), symbolic links will not be followed.
+  //! If On (the default), symbolic links will be followed.
   vtkSetMacro(FollowSymlinks, int);
   vtkBooleanMacro(FollowSymlinks, int);
   int GetFollowSymlinks() { return this->FollowSymlinks; }
